/*  %kw # %v    %n    %d    %t #                       */
/*  Version # 1    FNTIMAGE.C    11-Aug-93    10:14:02 #    */
/* *************************************************** */
/*   METAGRAPHICS SOFTWARE CORPORATION (c) 1987-1992   */
/*     MetaWINDOW graphics test program                */
/* *************************************************** */

#define TESTIT          /* define to make a test routine */

#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <ctype.h>
#include "metawndo.h"      /* master MetaWINDOW include file */

/* special stack size declaration if using Turbo or Borland C++ */
#ifdef      TurboC
extern unsigned _stklen = 14336U;  /* stack size 14K */
#endif


void ReadFontImage( fontRcd *, int, imageHeader * );
long int FontImageSize( fontRcd *, int );

void ReadFontImage( fontRcd *srcFont, int srcChar, imageHeader *dstImage )
/*
This routine reads the bitmap for character srcChar out of the font record
pointed to by srcFont, converts it to an image, and places at dstImage.
dstImage MUST be large enough to hold the resultant image. You can use
FontImageSize to compute the necessary image buffer size.
*/
{

    int i, row, shift;
    unsigned int *loc;
    unsigned char *imdata, *fontdata, *src;
    rect charRect;
    grafMap *fontBmap;


    /* if passed a null image buffer, don't do it! */
    if( dstImage == NULL )
        return;

    /* make a pointer to the fonts bitmap record */
    fontBmap = ( grafMap *) ( (long) srcFont + srcFont->grafMapTbl ); 

    /* make a pointer to the fonts location table */
    loc = ( unsigned int *) ( (long) srcFont + srcFont->locTbl ); 

    /* make a pointer to the font pixel data */
    fontdata = ( unsigned char *) ( (long) srcFont + srcFont->fontTbl ); 

    /* make a pointer to the image data buffer */
    imdata = ( ( unsigned char *) dstImage ) + sizeof( imageHeader );

    /* get a bounding rectangle for the character */
    i = srcChar - srcFont->minChar;
    SetRect( &charRect, loc[i], 0, loc[i+1], srcFont->chHeight );

    /* set the image header fields */
    dstImage->imWidth = charRect.Xmax - charRect.Xmin;
    dstImage->imHeight = charRect.Ymax - charRect.Ymin;
    dstImage->imAlign = 0;
    dstImage->imRowBytes = (dstImage->imWidth + 7) / 8;
    dstImage->imBits = (char) fontBmap->pixBits;
    dstImage->imPlanes = 1;

    /* get the data out of the font and into the image */

    /* calculate bits necessary to shift left */
    shift = charRect.Xmin % 8;

    /* for each row */
    for( row = 0; row < fontBmap->pixHeight; row++ ) {

        /* point to the font raster in question */
        src = fontdata + ( fontBmap->pixBytes * row );

        /* offset to first byte containing start of character */
        src +=  charRect.Xmin / 8;

        /* for each byte in a row */
        for( i = 0; i < dstImage->imRowBytes; i++ ) {

            /* shift it left and pack it into image data */
            *imdata = *src << shift;
            *imdata |= ( *(src+1) >> (8 - shift) );

            src++; imdata++;

        }

    }


}



long int FontImageSize( fontRcd *srcFont, int srcChar )
/*
This routine returns the number of bytes required to hold the image for
character srcChar in the font record pointed to by srcFont. This size
includes the image header structure
*/
{
    int i;
    unsigned int *loc;
    long fisize;

    /* make a pointer to the fonts location table */
    loc = ( unsigned int *) ( (long) srcFont + srcFont->locTbl ); 

    /* get an index into the location table for this character */
    i = srcChar - srcFont->minChar;

    /* compute number of bytes wide rounded to next word */  
    fisize = (long) ( (loc[i+1] - loc[i]) + 15 / 8 );
    fisize *= (long) srcFont->chHeight;
    fisize += (long) sizeof( imageHeader );

    return fisize;

}


#ifdef TESTIT

#include "conio.h"

void main( int, char *[] );
void QuitIt( int );

int GrafixCard, GrafixInput;

void main(int argc, char *argv[])
{

int i;
rect scrnR;
long fsize;
dirRec myDir;
fontRcd *myPatFont;
fontRcd *myCurFont;
imageHeader *myImage;
imageHeader *myImage2;
rect dR;


/* initialize */
MetQuery(argc,argv);
InitGraphics( GrafixCard );

/* go into graphics */
ScreenRect(&scrnR);
SetDisplay(GrafPg0);
InitMouse( GrafixInput ); 


/* ---- build a buffer for the pattern font file ---- */

/* find the size of the pattern file */
i = FileQuery( "pattern.fnt", &myDir, 1 );
if( i != 1 ) {
    MoveTo( 0, 10 );
    DrawString("can't find pattern file!");
    QuitIt(1);
}

i = ( int) ( myDir.fileSize + 10L);

/* allocate a buffer for it */ 
myPatFont = ( fontRcd * ) malloc( i );

/* read it into memory */
i = FileLoad( "pattern.fnt", myPatFont, i );
if( i < 0 ) {
    MoveTo( 0, 10 );
    DrawString("can't load pattern file!");
    QuitIt(2);
}



/* ---- build a buffer for the pattern image ---- */

/* find the size of character 5 in the pattern font */
fsize = FontImageSize( myPatFont, 5 );

/* allocate a buffer for it */
myImage = ( imageHeader * ) malloc( (int) fsize );
if( myImage == NULL ) {
    MoveTo( 0,10 );
    DrawString("not enough memory for pattern image buffer!");
    QuitIt(3);
}

/* read it into the image buffer */
ReadFontImage( myPatFont, 5, myImage );



/* ---- Lets play with Images --- do a WriteImage with it ---- */

MoveTo( 0,20 );
DrawString("WriteImage:");

SetRect( &dR, 50,50, 58, 58 );
WriteImage( &dR, ( image *) myImage );

/* ---- Lets play with patterns - define a pattern and do a pattern fill --- */
MoveTo( 0,100 );

DrawString( "Pattern Fill:");

DefinePattern( 10, ( patRcd *) myImage );

SetRect( &dR, 50, 150, 200, 300 );
FillRect( &dR, 10 );

free( myImage );
free( myPatFont );



/* ---- Lets play with cursors ---- */


/* find the size of the cursor file */
i = FileQuery( "cursor.fnt", &myDir, 1 );
if( i != 1 ) {
    MoveTo( 0, 10 );
    DrawString("can't find cursor file!");
    QuitIt(1);
}

i = ( int) ( myDir.fileSize + 10L );

/* allocate a buffer for it */ 
myCurFont = ( fontRcd * ) malloc( i );

/* read it into memory */
i = FileLoad( "cursor.fnt", myCurFont, i );
if( i < 0 ) {
    MoveTo( 0, 10 );
    DrawString("can't load cursor file!");
    QuitIt(2);
}

/* find the size of character 4 in the cursor font */
fsize = FontImageSize( myCurFont, 4 );

/* allocate a buffer for it */
myImage = ( imageHeader * ) malloc( (int) fsize );
if( myImage == NULL ) {
    MoveTo( 0, 10 );
    DrawString("not enough memory for cursor backcolor image buffer!");
    QuitIt(3);
}

/* find the size of character 5 in the cursor font */
fsize = FontImageSize( myCurFont, 5 );

/* allocate a buffer for it */
myImage2 = ( imageHeader * ) malloc( (int) fsize );
if( myImage2 == NULL ) {
    MoveTo( 0, 10 );
    DrawString("not enough memory for cursor forecolor image buffer!");
    QuitIt(3);
}


/* read in the images and re-define cursor 0 */
ReadFontImage( myCurFont, 4, myImage );
ReadFontImage( myCurFont, 5, myImage2 ); 
DefineCursor( 0, 8, 8, ( cursor *) myImage, ( cursor *) myImage2 ); 

CursorColor( Red, Green );
TrackCursor( True );
ShowCursor();

getch();

free( myImage );
free( myImage2 );
free( myCurFont );


QuitIt( 0 );

}  /* end MAIN */


void QuitIt( int exitcode )
{
    int i;

    SetDisplay(TextPg0);
    StopGraphics();
    i=QueryError();
    printf("QueryError=%d/%d",i >> 7,i & 127);
    exit( exitcode );
}

#include "metquery.c" 

#endif  /* TESTIT */

/* End of File - fntimage.C */

